'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defineProperty(obj, key, value) { return Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); }

var React = require('react');
var cx = require('./utils').cx;

function noop() {}

var Nav = React.createClass({
  displayName: 'Nav',

  mixins: [require('./InkBarMixin')],

  getInitialState: function getInitialState() {
    return {
      next: false,
      offset: 0,
      prev: false
    };
  },

  _getTabs: function _getTabs() {
    var _this = this;

    var props = this.props;
    var children = props.panels;
    var activeKey = props.activeKey;
    var rst = [];
    var prefixCls = props.prefixCls;

    React.Children.forEach(children, function (child) {
      var key = child.key;
      var cls = activeKey === key ? '' + prefixCls + '-tab-active' : '';
      cls += ' ' + prefixCls + '-tab';
      var events = {};
      if (child.props.disabled) {
        cls += ' ' + prefixCls + '-tab-disabled';
      } else {
        events = {
          onClick: _this.handleTabClick.bind(_this, key)
        };
      }
      var ref = {};
      if (activeKey === key) {
        ref.ref = 'activeTab';
      }
      rst.push(React.createElement(
        'div',
        _extends({}, events, {
          className: cls,
          key: key
        }, ref),
        React.createElement(
          'a',
          null,
          child.props.tab
        )
      ));
    });

    return rst;
  },

  handleTabClick: function handleTabClick(key) {
    this.props.handleTabClick(key);
  },

  componentDidMount: function componentDidMount() {
    this.componentDidUpdate();
  },

  componentDidUpdate: function componentDidUpdate(prevProps) {
    if (prevProps && prevProps.tabPosition !== this.props.tabPosition) {
      this.setOffset(0);
      return;
    }
    var navNode = React.findDOMNode(this.refs.nav);
    var navNodeWH = this.getOffsetWH(navNode);
    var navWrapNode = React.findDOMNode(this.refs.navWrap);
    var navWrapNodeWH = this.getOffsetWH(navWrapNode);
    var state = this.state;
    var offset = state.offset;
    if (navWrapNodeWH - offset < navNodeWH) {
      if (!state.next) {
        this.setState({
          next: true
        });
      }
    } else {
      var minOffset = navWrapNodeWH - navNodeWH;
      if (minOffset < 0 && minOffset > offset) {
        if (state.next) {
          this.setState({
            next: false
          });
        }
        this.setOffset(minOffset);
        offset = minOffset;
      }
    }
    if (offset < 0) {
      if (!state.prev) {
        this.setState({
          prev: true
        });
      }
    } else if (state.prev) {
      this.setState({
        prev: false
      });
    }
  },

  setOffset: function setOffset(offset) {
    offset = Math.min(0, offset);
    this.setState({
      offset: offset
    });
  },

  getOffsetWH: function getOffsetWH(node) {
    var tabPosition = this.props.tabPosition;
    var prop = 'offsetWidth';
    if (tabPosition === 'left' || tabPosition === 'right') {
      prop = 'offsetHeight';
    }
    return node[prop];
  },

  prev: function prev() {
    var navWrapNode = React.findDOMNode(this.refs.navWrap);
    var navWrapNodeWH = this.getOffsetWH(navWrapNode);
    var state = this.state;
    var offset = state.offset;
    this.setOffset(offset + navWrapNodeWH);
  },

  next: function next() {
    var navWrapNode = React.findDOMNode(this.refs.navWrap);
    var navWrapNodeWH = this.getOffsetWH(navWrapNode);
    var state = this.state;
    var offset = state.offset;
    this.setOffset(offset - navWrapNodeWH);
  },

  render: function render() {
    var props = this.props;
    var state = this.state;
    var prefixCls = props.prefixCls;
    var tabs = this._getTabs();
    var tabMovingDirection = props.tabMovingDirection;
    var tabPosition = props.tabPosition;
    var inkBarClass = '' + prefixCls + '-ink-bar';
    if (tabMovingDirection) {
      inkBarClass += ' ' + prefixCls + '-ink-bar-transition-' + tabMovingDirection;
    }
    var nextButton, prevButton;

    var showNextPrev = state.prev || state.next;

    if (showNextPrev) {
      var _cx, _cx2;

      prevButton = React.createElement(
        'span',
        {
          onClick: state.prev ? this.prev : noop,
          unselectable: 'unselectable',
          className: cx((_cx = {}, _defineProperty(_cx, '' + prefixCls + '-tab-prev', 1), _defineProperty(_cx, '' + prefixCls + '-tab-btn-disabled', !state.prev), _cx)) },
        React.createElement('span', { className: '' + prefixCls + '-tab-prev-icon' })
      );

      nextButton = React.createElement(
        'span',
        {
          onClick: state.next ? this.next : noop,
          unselectable: 'unselectable',
          className: cx((_cx2 = {}, _defineProperty(_cx2, '' + prefixCls + '-tab-next', 1), _defineProperty(_cx2, '' + prefixCls + '-tab-btn-disabled', !state.next), _cx2)) },
        React.createElement('span', { className: '' + prefixCls + '-tab-next-icon' })
      );
    }

    var navOffset = {};
    if (tabPosition === 'left' || tabPosition === 'right') {
      navOffset = {
        top: state.offset
      };
    } else {
      navOffset = {
        left: state.offset
      };
    }

    return React.createElement(
      'div',
      { className: '' + prefixCls + '-nav-container',
        style: props.style,
        ref: 'container' },
      prevButton,
      nextButton,
      React.createElement(
        'div',
        { className: '' + prefixCls + '-nav-wrap', ref: 'navWrap' },
        React.createElement(
          'div',
          { className: '' + prefixCls + '-nav-scroll' },
          React.createElement(
            'div',
            { className: '' + prefixCls + '-nav', ref: 'nav', style: navOffset },
            React.createElement('div', { className: inkBarClass, ref: 'inkBar' }),
            tabs
          )
        )
      )
    );
  }
});

module.exports = Nav;